@@ -1,6 +1,13 @@
 # Additional environment setting for rinse
 
-cmd_subst="
+# Add /usr/sbin and /sbin to PATH if chroot command can't be found
+if ! command -v chroot >/dev/null; then
+    PATH="${PATH:-/usr/bin:/bin}:/usr/sbin:/sbin"
+    export PATH
+fi
+
+# Set a list of command substitutions needed by rinse
+fakechroot_rinse_env_cmd_subst="
     /bin/mount=/bin/true
     /sbin/ldconfig=/bin/true
     /sbin/new-kernel-pkg=/bin/true
@@ -12,13 +19,12 @@ cmd_subst="
     /usr/sbin/libgcc_post_upgrade=/bin/true
     /usr/sbin/nscd=/bin/true
 "
+FAKECHROOT_CMD_SUBST="${FAKECHROOT_CMD_SUBST:+$FAKECHROOT_CMD_SUBST:}$(echo $fakechroot_rinse_env_cmd_subst | tr ' ' ':')"
 
-FAKECHROOT_EXCLUDE_PATH="${FAKECHROOT_EXCLUDE_PATH:+$FAKECHROOT_EXCLUDE_PATH:}/dev:/proc:/sys"
+# Set the default list of directories excluded from being chrooted
+FAKECHROOT_EXCLUDE_PATH="${FAKECHROOT_EXCLUDE_PATH:-/dev:/proc:/sys}"
+export FAKECHROOT_EXCLUDE_PATH
+
+# Change path for unix sockets because we don't want to exceed 108 bytes
 FAKECHROOT_AF_UNIX_PATH=/tmp
-FAKECHROOT_CMD_SUBST="${FAKECHROOT_CMD_SUBST:+$FAKECHROOT_CMD_SUBST:}$(echo $cmd_subst | tr ' ' ':')"
-export FAKECHROOT_EXCLUDE_PATH FAKECHROOT_AF_UNIX_PATH FAKECHROOT_CMD_SUBST
-
-if ! command -v chroot >/dev/null; then
-    PATH="${PATH:-/usr/bin:/bin}:/usr/sbin:/sbin"
-    export PATH
-fi
+export FAKECHROOT_AF_UNIX_PATH
