@@ -1,6 +1,6 @@
 /*
     libfakechroot -- fake chroot environment
-    Copyright (c) 2010 Piotr Roszatycki <dexter@debian.org>
+    Copyright (c) 2010, 2013 Piotr Roszatycki <dexter@debian.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -29,12 +29,11 @@
 /* Internal libc function */
 wrapper(__open64_2, int, (const char * pathname, int flags))
 {
-    char *fakechroot_path, fakechroot_buf[FAKECHROOT_PATH_MAX];
-
     debug("__open64_2(\"%s\", %d)", pathname, flags);
-    expand_chroot_path(pathname, fakechroot_path, fakechroot_buf);
-
+    expand_chroot_path(pathname);
     return nextcall(__open64_2)(pathname, flags);
 }
 
+#else
+typedef int empty_translation_unit;
 #endif
