@@ -1,6 +1,6 @@
 /*
     libfakechroot -- fake chroot environment
-    Copyright (c) 2010 Piotr Roszatycki <dexter@debian.org>
+    Copyright (c) 2010, 2013 Piotr Roszatycki <dexter@debian.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -25,8 +25,7 @@
 
 wrapper(acct, int, (const char * filename))
 {
-    char *fakechroot_path, fakechroot_buf[FAKECHROOT_PATH_MAX];
     debug("acct(\"%s\")", filename);
-    expand_chroot_path(filename, fakechroot_path, fakechroot_buf);
+    expand_chroot_path(filename);
     return nextcall(acct)(filename);
 }
