@@ -1,6 +1,6 @@
 /*
     libfakechroot -- fake chroot environment
-    Copyright (c) 2010 Piotr Roszatycki <dexter@debian.org>
+    Copyright (c) 2010, 2013 Piotr Roszatycki <dexter@debian.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,8 @@
 #include "strchrnul.h"
 #include "libfakechroot.h"
 #include "open.h"
-#include "unsetenv.h"
+#include "setenv.h"
+#include "readlink.h"
 
 
 /* Parse the FAKECHROOT_CMD_SUBST environment variable (the first
@@ -74,37 +75,33 @@ static int try_cmd_subst (char * env, const char * filename, char * cmd_subst)
 
 wrapper(execve, int, (const char * filename, char * const argv [], char * const envp []))
 {
+    int status;
     int file;
+    int is_base_orig = 0;
     char hashbang[FAKECHROOT_PATH_MAX];
     size_t argv_max = 1024;
     const char **newargv = alloca(argv_max * sizeof (const char *));
     char **newenvp, **ep;
     char *key, *env;
+    char tmpkey[1024], *tp;
     char *cmdorig;
     char tmp[FAKECHROOT_PATH_MAX];
     char substfilename[FAKECHROOT_PATH_MAX];
     char newfilename[FAKECHROOT_PATH_MAX];
     char argv0[FAKECHROOT_PATH_MAX];
     char *ptr;
-    unsigned int i, j, n, len, newenvppos;
+    unsigned int i, j, n, newenvppos;
     unsigned int do_cmd_subst = 0;
     size_t sizeenvp;
     char c;
-    char *fakechroot_path, fakechroot_buf[FAKECHROOT_PATH_MAX];
-    char *envkey[] = {
-        "FAKECHROOT",
-        "FAKECHROOT_BASE",
-        "FAKECHROOT_CMD_SUBST",
-        "FAKECHROOT_DEBUG",
-        "FAKECHROOT_DETECT",
-        "FAKECHROOT_EXCLUDE_PATH",
-        "FAKECHROOT_VERSION",
-        "LD_LIBRARY_PATH",
-        "LD_PRELOAD"
-    };
-    const int nr_envkey = sizeof envkey / sizeof envkey[0];
 
-    debug("execve(\"%s\", {\"%s\", ...}, {\"%s\", ...})", filename, argv[0], envp[0]);
+    char *elfloader = getenv("FAKECHROOT_ELFLOADER");
+    char *elfloader_opt_argv0 = getenv("FAKECHROOT_ELFLOADER_OPT_ARGV0");
+
+    if (elfloader && !*elfloader) elfloader = NULL;
+    if (elfloader_opt_argv0 && !*elfloader_opt_argv0) elfloader_opt_argv0 = NULL;
+
+    debug("execve(\"%s\", {\"%s\", ...}, {\"%s\", ...})", filename, argv[0], envp ? envp[0] : "(null)");
 
     strncpy(argv0, filename, FAKECHROOT_PATH_MAX);
 
@@ -117,53 +114,83 @@ wrapper(execve, int, (const char * filename, char * const argv [], char * const
 
     /* Scan envp and check its size */
     sizeenvp = 0;
-    for (ep = (char **)envp; *ep != NULL; ++ep) {
-        sizeenvp++;
+    if (envp) {
+        for (ep = (char **)envp; *ep != NULL; ++ep) {
+            sizeenvp++;
+        }
     }
 
     /* Copy envp to newenvp */
-    newenvp = malloc( (sizeenvp + 1) * sizeof (char *) );
+    newenvp = malloc( (sizeenvp + preserve_env_list_count + 1) * sizeof (char *) );
     if (newenvp == NULL) {
         __set_errno(ENOMEM);
         return -1;
     }
-    for (ep = (char **) envp, newenvppos = 0; *ep != NULL; ++ep) {
-        for (j = 0; j < nr_envkey; j++) {
-            len = strlen(envkey[j]);
-            if (strncmp(*ep, envkey[j], len) == 0 && (*ep)[len] == '=')
-                goto skip;
-        }
-        newenvp[newenvppos] = *ep;
-        newenvppos++;
-    skip: ;
-    }
-    newenvp[newenvppos] = NULL;
+    newenvppos = 0;
 
-    /* Add our variables to newenvp */
-    newenvp = realloc(newenvp, (newenvppos + nr_envkey + 1) * sizeof(char *));
+    /* Create new envp */
+    newenvp[newenvppos] = malloc(strlen("FAKECHROOT=true") + 1);
+    strcpy(newenvp[newenvppos], "FAKECHROOT=true");
+    newenvppos++;
 
-    if (newenvp == NULL) {
-        __set_errno(ENOMEM);
-        return -1;
-    }
-
-    for (j = 0; j < nr_envkey; j++) {
-        key = envkey[j];
+    /* Preserve old environment variables if not overwritten by new */
+    for (j = 0; j < preserve_env_list_count; j++) {
+        key = preserve_env_list[j];
         env = getenv(key);
-        if (env != NULL) {
+        if (env != NULL && *env) {
             if (do_cmd_subst && strcmp(key, "FAKECHROOT_BASE") == 0) {
                 key = "FAKECHROOT_BASE_ORIG";
+                is_base_orig = 1;
+            }
+            if (envp) {
+                for (ep = (char **) envp; *ep != NULL; ++ep) {
+                    strncpy(tmpkey, *ep, 1024);
+                    tmpkey[1023] = 0;
+                    if ((tp = strchr(tmpkey, '=')) != NULL) {
+                        *tp = 0;
+                        if (strcmp(tmpkey, key) == 0) {
+                            goto skip1;
+                        }
+                    }
+                }
             }
             newenvp[newenvppos] = malloc(strlen(key) + strlen(env) + 3);
             strcpy(newenvp[newenvppos], key);
             strcat(newenvp[newenvppos], "=");
             strcat(newenvp[newenvppos], env);
             newenvppos++;
+        skip1: ;
         }
     }
 
+    /* Append old envp to new envp */
+    if (envp) {
+        for (ep = (char **) envp; *ep != NULL; ++ep) {
+            strncpy(tmpkey, *ep, 1024);
+            tmpkey[1023] = 0;
+            if ((tp = strchr(tmpkey, '=')) != NULL) {
+                *tp = 0;
+                if (strcmp(tmpkey, "FAKECHROOT") == 0 ||
+                    (is_base_orig && strcmp(tmpkey, "FAKECHROOT_BASE") == 0))
+                {
+                    goto skip2;
+                }
+            }
+            newenvp[newenvppos] = *ep;
+            newenvppos++;
+        skip2: ;
+        }
+    }
+
+    newenvp[newenvppos] = NULL;
+
+    if (newenvp == NULL) {
+        __set_errno(ENOMEM);
+        return -1;
+    }
+
     if (do_cmd_subst) {
-        newenvp[newenvppos] = malloc(strlen("FAKECHROOT_CMD_ORIG=") + strlen(filename));
+        newenvp[newenvppos] = malloc(strlen("FAKECHROOT_CMD_ORIG=") + strlen(filename) + 1);
         strcpy(newenvp[newenvppos], "FAKECHROOT_CMD_ORIG=");
         strcat(newenvp[newenvppos], filename);
         newenvppos++;
@@ -174,11 +201,12 @@ wrapper(execve, int, (const char * filename, char * const argv [], char * const
     /* Exec substituded command */
     if (do_cmd_subst) {
         debug("nextcall(execve)(\"%s\", {\"%s\", ...}, {\"%s\", ...})", substfilename, argv[0], newenvp[0]);
-        return nextcall(execve)(substfilename, (char * const *)argv, newenvp);
+        status = nextcall(execve)(substfilename, (char * const *)argv, newenvp);
+        goto error;
     }
 
     /* Check hashbang */
-    expand_chroot_path(filename, fakechroot_path, fakechroot_buf);
+    expand_chroot_path(filename);
     strcpy(tmp, filename);
     filename = tmp;
 
@@ -195,8 +223,31 @@ wrapper(execve, int, (const char * filename, char * const argv [], char * const
     }
 
     /* No hashbang in argv */
-    if (hashbang[0] != '#' || hashbang[1] != '!')
-        return nextcall(execve)(filename, argv, newenvp);
+    if (hashbang[0] != '#' || hashbang[1] != '!') {
+        if (!elfloader) {
+            status = nextcall(execve)(filename, argv, newenvp);
+            goto error;
+        }
+
+        /* Run via elfloader */
+        for (i = 0, n = (elfloader_opt_argv0 ? 3 : 1); argv[i] != NULL && i < argv_max; ) {
+            newargv[n++] = argv[i++];
+        }
+
+        newargv[n] = 0;
+
+        n = 0;
+        newargv[n++] = elfloader;
+        if (elfloader_opt_argv0) {
+            newargv[n++] = elfloader_opt_argv0;
+            newargv[n++] = argv0;
+        }
+        newargv[n] = filename;
+
+        debug("nextcall(execve)(\"%s\", {\"%s\", \"%s\", ...}, {\"%s\", ...})", elfloader, newargv[0], newargv[n], newenvp[0]);
+        status = nextcall(execve)(elfloader, (char * const *)newargv, newenvp);
+        goto error;
+    }
 
     /* For hashbang we must fix argv[0] */
     hashbang[i] = hashbang[i+1] = 0;
@@ -208,7 +259,7 @@ wrapper(execve, int, (const char * filename, char * const argv [], char * const
             if (i > j) {
                 if (n == 0) {
                     ptr = &hashbang[j];
-                    expand_chroot_path(ptr, fakechroot_path, fakechroot_buf);
+                    expand_chroot_path(ptr);
                     strcpy(newfilename, ptr);
                 }
                 newargv[n++] = &hashbang[j];
@@ -221,11 +272,38 @@ wrapper(execve, int, (const char * filename, char * const argv [], char * const
 
     newargv[n++] = argv0;
 
-    for (i = 1; argv[i] != NULL && i<argv_max; ) {
+    for (i = 1; argv[i] != NULL && i < argv_max; ) {
         newargv[n++] = argv[i++];
     }
 
     newargv[n] = 0;
 
-    return nextcall(execve)(newfilename, (char * const *)newargv, newenvp);
+    if (!elfloader) {
+        status = nextcall(execve)(newfilename, (char * const *)newargv, newenvp);
+        goto error;
+    }
+
+    /* Run via elfloader */
+    j = elfloader_opt_argv0 ? 3 : 1;
+    if (n >= argv_max - 1) {
+        n = argv_max - j - 1;
+    }
+    newargv[n+j] = 0;
+    for (i = n; i >= j; i--) {
+        newargv[i] = newargv[i-j];
+    }
+    n = 0;
+    newargv[n++] = elfloader;
+    if (elfloader_opt_argv0) {
+        newargv[n++] = elfloader_opt_argv0;
+        newargv[n++] = argv0;
+    }
+    newargv[n] = newfilename;
+    debug("nextcall(execve)(\"%s\", {\"%s\", \"%s\", \"%s\", ...}, {\"%s\", ...})", elfloader, newargv[0], newargv[1], newargv[n], newenvp[0]);
+    status = nextcall(execve)(elfloader, (char * const *)newargv, newenvp);
+
+error:
+    free(newenvp);
+
+    return status;
 }
