@@ -1,6 +1,6 @@
 /*
     libfakechroot -- fake chroot environment
-    Copyright (c) 2010 Piotr Roszatycki <dexter@debian.org>
+    Copyright (c) 2010, 2013 Piotr Roszatycki <dexter@debian.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -23,16 +23,18 @@
 #ifdef HAVE_FCHOWNAT
 
 #define _ATFILE_SOURCE
+#include <sys/types.h>
 #include <sys/stat.h>
 #include "libfakechroot.h"
 
 
 wrapper(fchownat, int, (int dirfd, const char * path, uid_t owner, gid_t group, int flag))
 {
-    char *fakechroot_path, fakechroot_buf[FAKECHROOT_PATH_MAX];
     debug("fchownat(%d, \"%s\", %d, %d, %d)", dirfd, path, owner, group, flag);
-    expand_chroot_path(path, fakechroot_path, fakechroot_buf);
+    expand_chroot_path_at(dirfd, path);
     return nextcall(fchownat)(dirfd, path, owner, group, flag);
 }
 
+#else
+typedef int empty_translation_unit;
 #endif
