@@ -1,6 +1,6 @@
 /*
     libfakechroot -- fake chroot environment
-    Copyright (c) 2010 Piotr Roszatycki <dexter@debian.org>
+    Copyright (c) 2010, 2013 Piotr Roszatycki <dexter@debian.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -27,10 +27,11 @@
 
 wrapper(glob_pattern_p, int, (const char * pattern, int quote))
 {
-    char *fakechroot_path, fakechroot_buf[FAKECHROOT_PATH_MAX];
     debug("glob_pattern_p(\"%s\", %d)", pattern, quote);
-    expand_chroot_path(pattern, fakechroot_path, fakechroot_buf);
+    expand_chroot_path(pattern);
     return nextcall(glob_pattern_p)(pattern, quote);
 }
 
+#else
+typedef int empty_translation_unit;
 #endif
