@@ -1,6 +1,6 @@
 /*
     libfakechroot -- fake chroot environment
-    Copyright (c) 2010 Piotr Roszatycki <dexter@debian.org>
+    Copyright (c) 2010, 2013 Piotr Roszatycki <dexter@debian.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -20,14 +20,14 @@
 
 #include <config.h>
 
+#include <sys/types.h>
 #include <sys/stat.h>
 #include "libfakechroot.h"
 
 
 wrapper(lchown, int, (const char * path, uid_t owner, gid_t group))
 {
-    char *fakechroot_path, fakechroot_buf[FAKECHROOT_PATH_MAX];
     debug("lchown(\"%s\", %d, %d)", path, owner, group);
-    expand_chroot_path(path, fakechroot_path, fakechroot_buf);
+    expand_chroot_path(path);
     return nextcall(lchown)(path, owner, group);
 }
