@@ -1,6 +1,6 @@
 /*
     libfakechroot -- fake chroot environment
-    Copyright (c) 2010 Piotr Roszatycki <dexter@debian.org>
+    Copyright (c) 2010, 2013 Piotr Roszatycki <dexter@debian.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -25,8 +25,7 @@
 
 wrapper(tempnam, char *, (const char * dir, const char * pfx))
 {
-    char *fakechroot_path, fakechroot_buf[FAKECHROOT_PATH_MAX];
     debug("tempnam(\"%s\", \"%s\")", dir, pfx);
-    expand_chroot_path(dir, fakechroot_path, fakechroot_buf);
+    expand_chroot_path(dir);
     return nextcall(tempnam)(dir, pfx);
 }
