@@ -1,3 +1,4 @@
+#define _BSD_SOURCE
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fts.h>
@@ -23,7 +24,7 @@ int main (int argc, char *argv[]) {
     options = atoi(argv[1]);
     paths = argv + 2;
 
-    if ((tree = fts_open(paths, 0, 0)) == NULL) {
+    if ((tree = fts_open(paths, options, 0)) == NULL) {
         perror("fts_open");
         exit(1);
     }
