@@ -1,7 +1,10 @@
+#define _BSD_SOURCE
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <fcntl.h>
 
 int main (int argc, char *argv[]) {
     struct stat stab;
